//
//  Scanbot Barcode Scanner SDK Flutter Plugin
//
//  Copyright (c) 2020 doo GmbH. All rights reserved.
//  https://scanbot.io/sdk
//

import Foundation
import Flutter
import ScanbotBarcodeScannerSDK

class SDKCallHandler {
    func handle(call: FlutterMethodCall, rootViewController: UIViewController?, result: @escaping FlutterResult) {
        switch call.method {
        case "initScanbotSdk": self.handleInitSDK(call: call, result: result)
        case "getLicenseStatus": self.handleLicenseStatus(result: result)
        case "cleanupBarcodeStorage": self.handleClearBarcodeImagesStorage(result: result)

        case "startBarcodeScanner": self.handleBarcodeScanner(call: call, presenter: rootViewController, result: result)
        case "detectFromImageFile": self.handleDetectFromImageFile(call: call, result: result)
        default: self.handleUnknownCallError(call: call, result: result)
        }
    }
    
    private func handleLicenseStatus(result: @escaping FlutterResult) {
        let date = ScanbotSDK.licenseExpirationDate()
        let isValid = ScanbotSDK.isLicenseValid()
        let status = ScanbotSDK.licenseStatus()
        
        let statusResult = LicenseStatus(isLicenseValid: isValid, status: status, expirationDate: date)
        result(statusResult.encodedString)
    }
    
    private func handleClearBarcodeImagesStorage(result: @escaping FlutterResult) {
        SBSDKUIBarcodeImageStorage.cleanUp()
        result(true)
    }
    
    private func handleInitSDK(call: FlutterMethodCall, result: @escaping FlutterResult) {
        guard let arguments = call.arguments as? [String: Any],
            let config = SDKInitializationConfig(from: arguments) else {
                self.handleArgumentsError(call: call, result: result)
                return
        }
        ScanbotSDKInitializer.initialize(config: config, result: result)
    }
    
    private func handleDetectFromImageFile(call: FlutterMethodCall, result: @escaping FlutterResult) {
        guard let arguments = call.arguments as? [String: Any],
            let inputPath = arguments["uri"] as? String,
            let url = URL(string: inputPath)?.path,
            let image = UIImage(contentsOfFile: url) else {
            self.handleArgumentsError(call: call, result: result)
            return
        }
        
        let barcodeTypes = SBSDKBarcodeType.fromArguments(arguments) ?? []
        
        let detector = SBSDKBarcodeScanner(types: barcodeTypes)
        if let enableHighSensitivityMode = arguments["enableHighSensitivityMode"] as? Bool {
            detector.enableHighSensitivityMode = enableHighSensitivityMode
        }
        let barcodeResults = detector.detectBarCodes(on: image) ?? []
        let barcodeList = barcodeResults.map({ $0.encodedDictionary })
        let encodedResult: [String: Any?] = ["barcodeItems": barcodeList]

        result(encodedResult.encodedString)
    }
        
    private lazy var barcodeScannerPresenter = BarcodeScannerPresenter()
    private func handleBarcodeScanner(call: FlutterMethodCall,
                                      presenter: UIViewController?,
                                      result: @escaping FlutterResult) {
        guard let arguments = call.arguments as? [String: Any] else {
            self.handleArgumentsError(call: call, result: result)
            return
        }
        let config = SBSDKUIMachineCodeScannerConfiguration(from: arguments)
        let codes = SBSDKBarcodeType.fromArguments(arguments)
        
        if let presenter = presenter {
            self.barcodeScannerPresenter.present(on: presenter, with: config, barcodeTypes: codes, result: result)
        }
    }
    
    private func handleArgumentsError(call: FlutterMethodCall, result: @escaping FlutterResult) {
        result(FlutterError(code: "1001",
                            message: "ERROR_iOS: \(call.method) was called with wrong arguments",
                            details: call.arguments))
    }
    
    private func handleUnknownCallError(call: FlutterMethodCall, result: @escaping FlutterResult) {
        result(FlutterError(code: "1002",
                            message: "ERROR_iOS: \(call.method) is unknown or can't be handled",
                            details: nil))
    }
}
