import 'dart:async';
import 'dart:convert';

import 'package:flutter/services.dart';
import 'package:barcode_scanner/scanbot_sdk_models.dart';
import 'package:barcode_scanner/sdk_utils.dart';

import 'barcode_scanning_data.dart';

class ScanbotBarcodeSdk {
  static const MethodChannel _channel = const MethodChannel('barcode_scanner');

  /// Initializes the Scanbot SDK with given [config]s.
  /// The Scanbot SDK must be initialized before using any other API methods.
  static initScanbotSdk(ScanbotSdkConfig config) async {
    try {
      var arguments = {};
      addParam(config.licenseKey, "licenseKey", arguments);
      addParam(config.loggingEnabled, "loggingEnabled", arguments);
      addParam(config.storageBaseDirectory, "storageBaseDirectory", arguments);
      await _channel.invokeMethod("initScanbotSdk", arguments);
    } catch (e) {
      print(e);
      throw e;
    }
  }

  /// Returns the current license status of the Scanbot SDK.
  static Future<SdkLicenseStatus> getLicenseStatus() async {
    try {
      var licenseStatus = await _channel.invokeMethod("getLicenseStatus", {});
      var json = jsonDecode(licenseStatus);
      return SdkLicenseStatus.fromJson(json);
    } catch (e) {
      print(e);
      throw e;
    }
  }

  /// Clear storage folder with barcode snapped images
  static Future cleanupBarcodeStorage() async {
    try {
      await _channel.invokeMethod("cleanupBarcodeStorage", {});
      return;
    } catch (e) {
      print(e);
      throw e;
    }
  }

  /// Starts the Ready-to-Use Barcode Scanner UI as full-screen modal UI dialog.
  /// The Barcode Scanner is able to recognize 1D- and 2D-barcodes (QR codes, etc).
  static Future<BarcodeScanningResult> startBarcodeScanner(
      BarcodeScannerConfiguration config) async {
    try {
      String barcodeScanningResult =
          await _channel.invokeMethod("startBarcodeScanner", config.toJson());
      return BarcodeScanningResult.fromJson(jsonDecode(barcodeScanningResult));
    } catch (e) {
      return BarcodeScanningResult.error(e.toString());
    }
  }

  /// Detect barcodes  from image file uri
  /// The Barcode Scanner is able to recognize 1D- and 2D-barcodes (QR codes, etc).
  ///image [rotation]  between 0-360 degrees with 90 degree step
  /// [barcodeFormats] Format of barcode which  need to recognize
  static Future<BarcodeScanningResult> detectFromImageFile(
      Uri file,
      List<BarcodeFormat> barcodeFormats,
      bool enableHighSensitivityMode) async {
    try {
      var detectFromImageParams = DetectFromImageParams(
          file, barcodeFormats, enableHighSensitivityMode);
      String barcodeScanningResult = await _channel.invokeMethod(
          "detectFromImageFile", detectFromImageParams.toJson());
      return BarcodeScanningResult.fromJson(jsonDecode(barcodeScanningResult));
    } catch (e) {
      return BarcodeScanningResult.error(e.toString());
    }
  }
}
